#include "IO/Socket/BaseUDP.h"

namespace lwe
{
	BaseUDP::BaseUDP()
		: addrType(AddrType::IPV6), port(0), bound(false)
	{
	}

	BaseUDP::BaseUDP(const AddrType addrType)
		: addrType(addrType), port(0), bound(false)
	{
	}

	BaseUDP::BaseUDP(BaseUDP&& udp) noexcept
		: addrType(udp.addrType), address(std::move(udp.address)), port(udp.port), bound(true)
	{
		udp.addrType = AddrType::IPV6;
		udp.port = 0;
		udp.bound = false;
	}

	BaseUDP::BaseUDP(const BaseUDP& udp)
		: addrType(udp.addrType), address(udp.address), port(udp.port), bound(false)
	{
	}

	BaseUDP& BaseUDP::operator=(BaseUDP&& udp) noexcept
	{
		if (this == &udp)
			return *this;

		addrType = udp.addrType;
		address = std::move(udp.address);
		port = udp.port;
		bound = udp.bound;

		udp.addrType = AddrType::IPV6;
		udp.port = 0;
		udp.bound = false;

		return *this;
	}

	BaseUDP& BaseUDP::operator=(const BaseUDP& udp)
	{
		if (this == &udp)
			return *this;

		addrType = udp.addrType;
		address = udp.address;
		port = udp.port;
		bound = false;

		return *this;
	}

	bool BaseUDP::IsBound() const
	{
		return bound;
	}

	AddrType BaseUDP::GetAddressType() const
	{
		return addrType;
	}

	Str_8 BaseUDP::GetLocalAddress() const
	{
		return address;
	}

	UInt_16 BaseUDP::GetLocalPort() const
	{
		return port;
	}
}